/************************************************************/
/* mail: King_of_Freaks@ist-einmalig.de                     */
/* ICQ:  344921113                                          */
/************************************************************/
/* Grundfunktionen fr die Kommunikation mit dem CAN-Bus    */
/* und dem MSP430. Der Code Funktioniert soweit (senden &   */
/* empfangen) was jedoch nicht heit, das er frei von       */
/* fehlern ist. Hast du einen Fehler gefunden? Ja? Dann     */
/* sei so gut und informiere mich bitte.                    */
/************************************************************/
/* Getestet mit:                                            */
/* MSP430F149 mit 8MHz Quarz (SPI bei 1 MHz)                */
/* MCP2510-I/SO mit 2MHz Quarz (kompatibel zu MCP2515)      */
/* SN65HVD232D (1MBit 3,3V Transreciver)                    */
/* MAX3233EEWP (RS232 3,3V  Transreciver)                   */
/************************************************************/
/* bugs:                                                    */
/* - zur zeit gehen nur daten ber den rx1 ein. (es wird    */
/*   wohl an der initialisierung liegen)                    */
/* - befinden sich 2. oder mehr Frames auf dem Bus, so wird */
/*   ein Interrupt ausgelst, den ich noch nicht            */
/*   interpretieren konnte. In diesem Fall wird NICHTS      */
/*   in die RX Buffer geschrieben und der Interrupt muss    */
/*   gelscht werden! (Kollision von Daten?)                */
/************************************************************/


#include <msp430x14x.h>
#include "mcp_reg.h"
#include "tools.h"


/*schreibt ein byte an eine addresse.*/
void write_mcp(byte addr, byte value)
{
	  P2OUT &= ~0x01;             //cs-low->enable
	  TXBUF0 = 0x02;              //schreibbefehl
	  while (!(UTCTL0 & TXEPT));  //warte bist bermittelt	
	  TXBUF0 = addr;              //adresse
	  while (!(UTCTL0 & TXEPT));  //warte bist bermittelt
	  TXBUF0 = value;             //daten
	  while (!(UTCTL0 & TXEPT));  //warte bist bermittelt
	  P2OUT |= 0x01;              //cs-high->disable
	  
	  wait(10);
}

/*softwarereset fr den mcp2510/2515*/
void reset_mcp(void)
{
	  P2OUT &= ~0x01;		//cs-low->enable
	  TXBUF0 = 0xc0;		//resetcomando
	  while (!(UTCTL0 & TXEPT));	//warte bist bermittelt
	  P2OUT |= 0x01; 		//cs-hign->disable
	  
	  wait(10);
}

/*liesst daten aus dem register*/
byte read_mcp(byte addr)
{
	  byte data;		
	  P2OUT &= ~0x01;             //cs-low->enable
	  TXBUF0 = 0x03;              //lesebefehl senden
	  while (!(UTCTL0 & TXEPT));  //warte bist bermittelt
	  TXBUF0 = addr;              //adresse senden
	  while (!(UTCTL0 & TXEPT));  //warte bist bermittelt
	  TXBUF0 = 0xaa;              //dummybyte senden
	  while (!(UTCTL0 & TXEPT));  //warte bist bermittelt
	  data = RXBUF0;	      //daten aus dem uartbuffer holen und in var. data schreiben	
	  P2OUT |= 0x01;              //cs-high->disable
	
	  wait(10);
	  
	  return data; 			
}

/*setzt den mode des mcp2510/2515*/
void can_mode(byte mode)	
{
  	write_mcp(CANCTRL,mode);	//schreibt auf das kontrollregister den mode
}

/*schreibt viele hintereinander liegende register auf einmal ab reg. anzahl der zu schreibenden regs mit len bergeben*/
void can_write_many_registers(byte reg, byte len,byte *dta)
{
	  byte x;
	  
	  P2OUT &= ~0x01;             //cs-low->enable
	  TXBUF0 = 0x02;              //schreibbefehl
	  while (!(UTCTL0 & TXEPT));  //warte bist bermittelt
	  TXBUF0 = reg;		      //schreibe reg auf das uartregister		
	  while (!(UTCTL0 & TXEPT));  //warte bist bermittelt
	  for( x=0;x<len;x++)	      //solange x < datenlnge	
	  {
		    TXBUF0 = (*dta);		//schreibe dta auf das uartgerister
		    while (!(UTCTL0 & TXEPT));	//warte bist bermittelt
		    dta++;			//nchstes byte
	  }
	  P2OUT |= 0x01;              //cs-high->disable
	  wait(10);
}

/*setzt die id des can-controllers*/
void can_write_id(byte mcp_addr, byte ext, dword can_id)
{
	  word canid;
	  byte tbufdata[4];
	  
	  canid = (word)(can_id & 0x0ffff);
	  if(ext == 1)
	  {
		    tbufdata[EID0] = (byte) (canid & 0xff);
		    tbufdata[EID8] = (byte) (canid / 256);
		    canid = (word)(can_id / 0x10000);
		    tbufdata[SIDL] = (byte) (canid & 0x03);
		    tbufdata[SIDL] +=  (byte)((canid & 0x1c)*8);
		    tbufdata[SIDL] |= TXBnSIDL_EXIDE;
		    tbufdata[SIDH] = (byte)(canid / 32);
	  }
	  else
	  {
		    tbufdata[SIDH] = (byte)(canid / 8);
		    tbufdata[SIDL] = (byte)((canid & 0x07)*32);
		    tbufdata[EID0] = 0;
		    tbufdata[EID8] = 0;
	  }
	  can_write_many_registers(mcp_addr, 4, tbufdata);
	  wait(10);
}

/*initialisiert den mcp2510/2515*/
void can_init(void)
{
  	reset_mcp();
  	reset_mcp();
  	reset_mcp();
  	can_mode(0x80);               //mode: configuration
  
  	write_mcp(CNF1,0x00);		//
  	write_mcp(CNF2,0x90);		//bei 2MHz -> 125kboud
  	write_mcp(CNF3,0x02);		//	
	
	write_mcp(TXB0CTRL, 0x00);
	write_mcp(TXB1CTRL, 0x00);
	write_mcp(TXB2CTRL, 0x00);
	
	write_mcp(RXM0SIDH, 0x00);
	write_mcp(RXM0SIDL, 0x00);
	
	write_mcp(RXM1SIDH, 0xff);
	write_mcp(RXM1SIDL, 0xf3);
	
	write_mcp(RXF0SIDH, 0xff);
	write_mcp(RXF0SIDL, 0xe0);
	
	
	write_mcp(RXF1SIDH, 0xff);
	write_mcp(RXF1SIDL, 0xe0);
	
	write_mcp(RXF2SIDH, 0xff);
	write_mcp(RXF2SIDL, 0xe0);
	
	write_mcp(RXF3SIDH, 0xff);
	write_mcp(RXF3SIDL, 0xe0);
	
	write_mcp(RXF4SIDH, 0xff);
	write_mcp(RXF4SIDL, 0xe0);
	
	write_mcp(RXF5SIDH, 0xff);
	write_mcp(RXF5SIDL, 0xe0);
	
	write_mcp(RXB0CTRL, 0x64);
	
	write_mcp(RXB1CTRL, 0x64);
	
	write_mcp(CANINTE, 0x3f);
	
	write_mcp(CANINTF, 0x0);
	
	write_mcp(EFLAG, 0x00);
	
	write_mcp(TXB0SIDL, 0x40);
	
	write_mcp(TXB1SIDH, 0x60);
	
	write_mcp(CANCTRL, 0x00);
	
  	can_mode(0x00);     //normal mode
	wait(50000);
}
	
	

/*liet den status des mcp2510/2515*/
byte can_state(void)
{
  	return (read_mcp(MCP_READ_STATUS));
}

/*checkt die interruptregister und liefert ihren wert*/
byte can_chk(void)
{
	  byte buffer;
	  buffer = 0;
	  buffer = read_mcp(CANINTF);		//liet das interruptregister aus
	  wait(10);
	  return (buffer & 0x03);
}

/*liet die anzahl der register (len) hintereinander weg ab der adresse regaddr*/
void can_read_many_registers(byte regaddr, byte len, byte *dta)
{
	  byte x;
	  P2OUT &= ~0x01;             //cs-low->enable
	  TXBUF0 = 0x02;              //schreibbefehl
	  while (!(UTCTL0 & TXEPT));  //warte bist bermittelt
	
	  TXBUF0 = regaddr;	      //schreibt die registeraddresse auf den uart	
	  while (!(UTCTL0 & TXEPT));  //warte bist bermittelt
	  
	  for(x=1;x<len;x++)	//solange x < datengre	
	  {	
		    *dta = RXBUF0;
		    dta++;   
	  }
	  
	  P2OUT |= 0x01;              //cs-high->disable
	  wait(10);
}

/*liesst die id des empfangenen frames aus*/
void can_read_id(byte mcp_addr, byte* ext, dword* can_id)
{
	byte tbufdata[2];
	tbufdata[0] = (read_mcp(RXB0SIDL)>>5);
	tbufdata[1] = (read_mcp(RXB0SIDH)<<3);
	*can_id = tbufdata[1] | tbufdata[0];
}	

/*modifiziert ein register*/
void can_modify_register(byte regaddr, byte mask, byte regvalue)
{
	  P2OUT &= ~0x01;             //cs-low->enable
	  TXBUF0 = 0x05;              //MCP_BITMOD
	  while (!(UTCTL0 & TXEPT));
	  TXBUF0 = regaddr;
	  while (!(UTCTL0 & TXEPT));
	  TXBUF0 = mask;
	  while (!(UTCTL0 & TXEPT));
	  TXBUF0 = regvalue;
	  while (!(UTCTL0 & TXEPT));
	  P2OUT |= 0x01;              //cs-high->disable
	  
	  wait(10);
}

/*lscht alle interrupts des mcp2510/2515*/
void can_int_clear(void)
{
  	write_mcp(CANINTF, CANINTF_ALL_DISABLE);
}

/*lscht den rx0 interrupt*/
void can_clear_rx0(void)
{
  	can_modify_register(CANINTF,RX0IF,0x00);
}
/*lscht den rx1 interrupt*/
void can_clear_rx1(void)
{
  	can_modify_register(CANINTF,RX1IF,0x00);
}

/*schickt ein frame an eine bestimmte adresse mit max 8 datenbytes ber tx0*/
void can_tx0(dword can_id, byte ext_id, byte dlc, byte *dta)
{
	  if(dlc > 8)
	    	return;
	  can_write_id(TXB0SIDH,ext_id,can_id);
	  write_mcp(TXB0DLC,dlc);
	  can_write_many_registers(TXB0D0,dlc,dta);
	  write_mcp(TXB0CTRL,0x0b);
}

/*schickt ein frame an eine bestimmte adresse ber tx1*/
void can_tx1(dword can_id, byte ext_id, byte dlc, byte *dta)
{
	  can_write_id(TXB1SIDH,ext_id,can_id);
	  write_mcp(TXB1DLC,dlc);
	  can_write_many_registers(TXB1D0,dlc,dta);
	  write_mcp(TXB1CTRL,0x0b);
}
/*schickt ein frame an eine bestimmte adresse ber tx2*/
void can_tx2(dword can_id, byte ext_id, byte dlc, byte *dta)
{
	  can_write_id(TXB2SIDH,ext_id,can_id);
	  write_mcp(TXB2DLC,dlc);
	  can_write_many_registers(TXB2D0,dlc,dta);
	  write_mcp(TXB2CTRL,0x0b);
}

/*holt die id und die lnge des aktuellen frames aus rx0*/
void can_rx0(dword *can_id, byte *ext_id, byte *dlc)
{
	  can_read_id(RXB0SIDH, ext_id, can_id);
	  *dlc = read_mcp(RXB0DLC);
}

/*holt die id und die lnge des aktuellen frames aus rx1*/
void can_rx1(dword *can_id, byte *ext_id, byte *dlc)
{
	can_read_id(RXB1SIDH, ext_id, can_id);
  	*dlc = read_mcp(RXB1DLC);
}


