/************************************************************/
/* mail: King_of_Freaks@ist-einmalig.de                     */
/* ICQ:  344921113                                          */
/************************************************************/
/* Grundfunktionen fr die Kommunikation mit dem CAN-Bus    */
/* und dem MSP430. Der Code Funktioniert soweit (senden &   */
/* empfangen) was jedoch nicht heit, das er frei von       */
/* fehlern ist. Hast du einen Fehler gefunden? Ja? Dann     */
/* sei so gut und informiere mich bitte.                    */
/************************************************************/
/* Getestet mit:                                            */
/* MSP430F149 mit 8MHz Quarz (SPI bei 1 MHz)                */
/* MCP2510-I/SO mit 2MHz Quarz (kompatibel zu MCP2515)      */
/* SN65HVD232D (1MBit 3,3V Transreciver)                    */
/* MAX3233EEWP (RS232 3,3V  Transreciver)                   */
/************************************************************/
/* bugs:                                                    */
/* - zur zeit gehen nur daten ber den rx1 ein. (es wird    */
/*   wohl an der initialisierung liegen)                    */
/* - befinden sich 2. oder mehr Frames auf dem Bus, so wird */
/*   ein Interrupt ausgelst, den ich noch nicht            */
/*   interpretieren konnte. In diesem Fall wird NICHTS      */
/*   in die RX Buffer geschrieben und der Interrupt muss    */
/*   gelscht werden! (Kollision von Daten?)                */
/************************************************************/

#include <stdio.h>
#include <msp430x14x.h>

#include "can_new.h"
#include "hw-init.h"
#include "tools.h"
#include "MCP_Reg.h"


unsigned char can_array[8];

void dac(byte val)
{	
	P4OUT = val;
}

void can_read_recv(byte *dta)
{
	byte i;
	for(i=0;i<8;++i)
	{
		dta[i] = read_mcp(RXB0D0+i);
	}

}

int main( void )
{
	byte extid;
	byte dlc;
	dword canid;
	byte status;
	
	P1DIR = 0xff;
	P1SEL = 0x00;
	P1IE = 0x00;
	
	P4DIR = 0xff;
	P4SEL = 0x00;
	P4OUT = 0x00;
	
	P5DIR = 0xff;
	P5SEL = 0x00;
	P5OUT = 0x00;
	
	P2SEL = 0x00;
  	P2DIR = 0x01;
  	P2OUT = 0xFF;
  	
	init_spi();
	init_rs232();
	can_init();
	
	for(;;)
	{
		status = can_chk();		//stellt fest ob etwas empfangen worden ist -> gibt den resiver aus
		if(status)
		{
			switch(status)
			{
				case RX0IF:
					can_rx0(&canid,&extid,&dlc);	//ermittelt aus dem empfangenem frame die id und datenlnge
					can_read_recv(can_array);	//holt die daten aus dem register
					if(canid == 0x22)	
					{
						dac(can_array[0]);
					}
					if(canid == 0x29)
					{
						P1OUT = can_array[1];
					}
					P5OUT = can_array[2];
					
  					can_tx0(0x66, 0x00, 8, can_array);	//sendet das empfangene frame mit der id 0x66 zurck
					can_clear_rx0();			//lscht die register des rx0
				break;
				
				case RX1IF:
					can_rx1(&canid,&extid,&dlc);	//ermittelt aus dem empfangenem frame die id und datenlnge
					can_read_recv(can_array);	//holt die daten aus dem register
					if(canid == 0x22)	
					{
						dac(can_array[0]);
					}
					if(canid == 0x29)
					{
						P1OUT = can_array[1];
					}
					P5OUT = can_array[2];
					
  					can_tx1(0x77, 0x00, 8, can_array);	//sendet das empfangene frame mit der id 0x77 zurck
					can_clear_rx1();			//lscht die register des rx0
				break;
				default:
					sout("\n\runknown status\n\r");
					can_int_clear();
				break;
					
			}
		}
	}
	return 0;
}
